#include <mycpp/util.h>
#include <mycpp/io.h>

#include "mconfig_pargen.h"

using namespace MyCpp;
using namespace Pargen;

namespace MConfig {

static void
print_whsp (File *file,
            Size num_spaces)
{
    for (Size i = 0; i < num_spaces; i++)
        file->print (" ");
}

static void
print_tab (File *file,
           Size nest_level)
{
    print_whsp (file, nest_level * 1);
}

static Ref<Grammar> create_mconfig_Word ();
static Ref<Grammar> create_mconfig_Value ();
static Ref<Grammar> create_mconfig_Key ();
static Ref<Grammar> create_mconfig_Option ();
static Ref<Grammar> create_mconfig_SectionEntry ();
static Ref<Grammar> create_mconfig_SectionBody ();
static Ref<Grammar> create_mconfig_Section ();

static void
mconfig_Word_set_any_token (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);

    abortIf (__self->mconfig_element_type != MConfigElement::_Word);

    MConfig_Word * const &self = static_cast <MConfig_Word*> (__self);

    self->any_token = static_cast <ParserElement_Token*> (___el);
}

static ParserElement*
mconfig_Word_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Word))) MConfig_Word;
}

static Ref<Grammar>
create_mconfig_Word ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Word_creation_func));
    grammar->name = String::forData ("Word");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());

        Ref<Grammar_Immediate_SingleToken> grammar__immediate = grab (new Grammar_Immediate_SingleToken (""));

        bool mconfig_word_token_match_func (
                ConstMemoryDesc const &token,
                void                  *token_user_ptr,
                void                  *user_data);

        grammar__immediate->token_match_cb = mconfig_word_token_match_func;
        grammar__immediate->token_match_cb_name = grab (new String ("mconfig_word_token_match_func"));

        entry->grammar = grammar__immediate;
        entry->assignment_func = mconfig_Word_set_any_token;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Word (MConfig_Word const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Word\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("Word\n");

    print_tab (errf, nest_level + 1);
    errf->print ("token: \n");
}

static void
mconfig_Value_List_set_word (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Value ||
             __el->mconfig_element_type != MConfigElement::_Word);

    MConfig_Value * const &_self = static_cast <MConfig_Value*> (__self);

    abortIf (_self->value_type != MConfig_Value::_List);

    MConfig_Value_List * const &self = static_cast <MConfig_Value_List*> (_self);

    self->words.append (static_cast <MConfig_Word*> (__el));
}

static void
mconfig_Value_List_set_value (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Value ||
             __el->mconfig_element_type != MConfigElement::_Value);

    MConfig_Value * const &_self = static_cast <MConfig_Value*> (__self);

    abortIf (_self->value_type != MConfig_Value::_List);

    MConfig_Value_List * const &self = static_cast <MConfig_Value_List*> (_self);

    self->value = static_cast <MConfig_Value*> (___el);
}

static ParserElement*
mconfig_Value_List_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Value_List))) MConfig_Value_List;
}

static Ref<Grammar>
create_mconfig_Value_List ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Value_List_creation_func));
    grammar->name = String::forData ("Value_List");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->flags |= CompoundGrammarEntry::Optional;
        entry->flags |= CompoundGrammarEntry::Sequence;
        entry->grammar = create_mconfig_Word ();
        entry->assignment_func = mconfig_Value_List_set_word;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());

        Ref<Grammar_Immediate_SingleToken> grammar__immediate = grab (new Grammar_Immediate_SingleToken (","));
        entry->grammar = grammar__immediate;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->grammar = create_mconfig_Value ();
        entry->assignment_func = mconfig_Value_List_set_value;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Value_List (MConfig_Value_List const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Value_List\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("Value_List\n");

    {
        List<MConfig_Word*>::DataIterator sub_iter (el->words);
        while (!sub_iter.done ()) {
            MConfig_Word *&sub = sub_iter.next ();
            dump_mconfig_Word (sub, nest_level + 1);
        }
    }

    print_tab (errf, nest_level + 1);
    errf->print ("token: ,\n");
    dump_mconfig_Value (el->value, nest_level + 1);

}

static void
mconfig_Value_Word_set_word (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Value ||
             __el->mconfig_element_type != MConfigElement::_Word);

    MConfig_Value * const &_self = static_cast <MConfig_Value*> (__self);

    abortIf (_self->value_type != MConfig_Value::_Word);

    MConfig_Value_Word * const &self = static_cast <MConfig_Value_Word*> (_self);

    self->words.append (static_cast <MConfig_Word*> (__el));
}

static ParserElement*
mconfig_Value_Word_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Value_Word))) MConfig_Value_Word;
}

static Ref<Grammar>
create_mconfig_Value_Word ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Value_Word_creation_func));
    grammar->name = String::forData ("Value_Word");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->flags |= CompoundGrammarEntry::Optional;
        entry->flags |= CompoundGrammarEntry::Sequence;
        entry->grammar = create_mconfig_Word ();
        entry->assignment_func = mconfig_Value_Word_set_word;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Value_Word (MConfig_Value_Word const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Value_Word\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("Value_Word\n");

    {
        List<MConfig_Word*>::DataIterator sub_iter (el->words);
        while (!sub_iter.done ()) {
            MConfig_Word *&sub = sub_iter.next ();
            dump_mconfig_Word (sub, nest_level + 1);
        }
    }

}

static Ref<Grammar>
create_mconfig_Value ()
{
    static Ref<Grammar_Switch> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Switch ());
    grammar->name = String::forData ("Value");

    {
        Ref<SwitchGrammarEntry> entry = grab (new SwitchGrammarEntry ());
        entry->grammar = create_mconfig_Value_List ();

        grammar->grammar_entries.append (entry);
    }

    {
        Ref<SwitchGrammarEntry> entry = grab (new SwitchGrammarEntry ());
        entry->grammar = create_mconfig_Value_Word ();

        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Value (MConfig_Value const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Value\n");
        return;
    }

    switch (el->value_type) {
        case MConfig_Value::_List:
            dump_mconfig_Value_List (static_cast <MConfig_Value_List const *> (el), nest_level + 0);
            break;
        case MConfig_Value::_Word:
            dump_mconfig_Value_Word (static_cast <MConfig_Value_Word const *> (el), nest_level + 0);
            break;
        default:
            abortIfReached ();
    }
}

static void
mconfig_Key_set_word (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Key ||
             __el->mconfig_element_type != MConfigElement::_Word);

    MConfig_Key * const &self = static_cast <MConfig_Key*> (__self);

    self->words.append (static_cast <MConfig_Word*> (__el));
}

static ParserElement*
mconfig_Key_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Key))) MConfig_Key;
}

static Ref<Grammar>
create_mconfig_Key ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Key_creation_func));
    grammar->name = String::forData ("Key");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->flags |= CompoundGrammarEntry::Sequence;
        entry->grammar = create_mconfig_Word ();
        entry->assignment_func = mconfig_Key_set_word;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Key (MConfig_Key const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Key\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("Key\n");

    {
        List<MConfig_Word*>::DataIterator sub_iter (el->words);
        while (!sub_iter.done ()) {
            MConfig_Word *&sub = sub_iter.next ();
            dump_mconfig_Word (sub, nest_level + 1);
        }
    }

}

static void
mconfig_Option_KeyValue_set_key (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Option ||
             __el->mconfig_element_type != MConfigElement::_Key);

    MConfig_Option * const &_self = static_cast <MConfig_Option*> (__self);

    abortIf (_self->option_type != MConfig_Option::_KeyValue);

    MConfig_Option_KeyValue * const &self = static_cast <MConfig_Option_KeyValue*> (_self);

    self->key = static_cast <MConfig_Key*> (___el);
}

static void
mconfig_Option_KeyValue_set_value (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Option ||
             __el->mconfig_element_type != MConfigElement::_Value);

    MConfig_Option * const &_self = static_cast <MConfig_Option*> (__self);

    abortIf (_self->option_type != MConfig_Option::_KeyValue);

    MConfig_Option_KeyValue * const &self = static_cast <MConfig_Option_KeyValue*> (_self);

    self->value = static_cast <MConfig_Value*> (___el);
}

bool mconfig_accept_option (
        MConfig_Option *parser_element,
        Pargen::ParserControl *parser_control,
        void *data);

bool __pargen_mconfig_accept_option (
        ParserElement *parser_element,
        Pargen::ParserControl *parser_control,
        void *data)
{
    MConfigElement * const &_el =
            static_cast <MConfigElement*> (parser_element);
    abortIf (_el->mconfig_element_type != MConfigElement::_Option);

    MConfig_Option * const &el =
            static_cast <MConfig_Option*> (_el);

    return mconfig_accept_option (el, parser_control, data);
}

static ParserElement*
mconfig_Option_KeyValue_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Option_KeyValue))) MConfig_Option_KeyValue;
}

static Ref<Grammar>
create_mconfig_Option_KeyValue ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Option_KeyValue_creation_func));
    grammar->name = String::forData ("Option_KeyValue");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->grammar = create_mconfig_Key ();
        entry->assignment_func = mconfig_Option_KeyValue_set_key;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());

        Ref<Grammar_Immediate_SingleToken> grammar__immediate = grab (new Grammar_Immediate_SingleToken ("="));
        entry->grammar = grammar__immediate;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->grammar = create_mconfig_Value ();
        entry->assignment_func = mconfig_Option_KeyValue_set_value;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());

        Ref<Grammar_Immediate_SingleToken> grammar__immediate = grab (new Grammar_Immediate_SingleToken (";"));
        entry->grammar = grammar__immediate;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->inline_match_func = __pargen_mconfig_accept_option;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Option_KeyValue (MConfig_Option_KeyValue const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Option_KeyValue\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("Option_KeyValue\n");

    dump_mconfig_Key (el->key, nest_level + 1);

    print_tab (errf, nest_level + 1);
    errf->print ("token: =\n");
    dump_mconfig_Value (el->value, nest_level + 1);

    print_tab (errf, nest_level + 1);
    errf->print ("token: ;\n");
}

static void
mconfig_Option_Key_set_key (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Option ||
             __el->mconfig_element_type != MConfigElement::_Key);

    MConfig_Option * const &_self = static_cast <MConfig_Option*> (__self);

    abortIf (_self->option_type != MConfig_Option::_Key);

    MConfig_Option_Key * const &self = static_cast <MConfig_Option_Key*> (_self);

    self->key = static_cast <MConfig_Key*> (___el);
}

static ParserElement*
mconfig_Option_Key_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Option_Key))) MConfig_Option_Key;
}

static Ref<Grammar>
create_mconfig_Option_Key ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Option_Key_creation_func));
    grammar->name = String::forData ("Option_Key");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->grammar = create_mconfig_Key ();
        entry->assignment_func = mconfig_Option_Key_set_key;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());

        Ref<Grammar_Immediate_SingleToken> grammar__immediate = grab (new Grammar_Immediate_SingleToken (";"));
        entry->grammar = grammar__immediate;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->inline_match_func = __pargen_mconfig_accept_option;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Option_Key (MConfig_Option_Key const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Option_Key\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("Option_Key\n");

    dump_mconfig_Key (el->key, nest_level + 1);

    print_tab (errf, nest_level + 1);
    errf->print ("token: ;\n");
}

static ParserElement*
mconfig_Option_Newline_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Option_Newline))) MConfig_Option_Newline;
}

static Ref<Grammar>
create_mconfig_Option_Newline ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Option_Newline_creation_func));
    grammar->name = String::forData ("Option_Newline");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());

        Ref<Grammar_Immediate_SingleToken> grammar__immediate = grab (new Grammar_Immediate_SingleToken (";"));
        entry->grammar = grammar__immediate;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Option_Newline (MConfig_Option_Newline const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Option_Newline\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("Option_Newline\n");

    print_tab (errf, nest_level + 1);
    errf->print ("token: ;\n");
}

static Ref<Grammar>
create_mconfig_Option ()
{
    static Ref<Grammar_Switch> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Switch ());
    grammar->name = String::forData ("Option");

    {
        Ref<SwitchGrammarEntry> entry = grab (new SwitchGrammarEntry ());
        entry->grammar = create_mconfig_Option_KeyValue ();

        grammar->grammar_entries.append (entry);
    }

    {
        Ref<SwitchGrammarEntry> entry = grab (new SwitchGrammarEntry ());
        entry->grammar = create_mconfig_Option_Key ();

        grammar->grammar_entries.append (entry);
    }

    {
        Ref<SwitchGrammarEntry> entry = grab (new SwitchGrammarEntry ());
        entry->grammar = create_mconfig_Option_Newline ();

        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Option (MConfig_Option const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Option\n");
        return;
    }

    switch (el->option_type) {
        case MConfig_Option::_KeyValue:
            dump_mconfig_Option_KeyValue (static_cast <MConfig_Option_KeyValue const *> (el), nest_level + 0);
            break;
        case MConfig_Option::_Key:
            dump_mconfig_Option_Key (static_cast <MConfig_Option_Key const *> (el), nest_level + 0);
            break;
        case MConfig_Option::_Newline:
            dump_mconfig_Option_Newline (static_cast <MConfig_Option_Newline const *> (el), nest_level + 0);
            break;
        default:
            abortIfReached ();
    }
}

static void
mconfig_SectionEntry_Parameter_set_option (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_SectionEntry ||
             __el->mconfig_element_type != MConfigElement::_Option);

    MConfig_SectionEntry * const &_self = static_cast <MConfig_SectionEntry*> (__self);

    abortIf (_self->section_entry_type != MConfig_SectionEntry::_Parameter);

    MConfig_SectionEntry_Parameter * const &self = static_cast <MConfig_SectionEntry_Parameter*> (_self);

    self->option = static_cast <MConfig_Option*> (___el);
}

static ParserElement*
mconfig_SectionEntry_Parameter_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_SectionEntry_Parameter))) MConfig_SectionEntry_Parameter;
}

static Ref<Grammar>
create_mconfig_SectionEntry_Parameter ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_SectionEntry_Parameter_creation_func));
    grammar->name = String::forData ("SectionEntry_Parameter");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->grammar = create_mconfig_Option ();
        entry->assignment_func = mconfig_SectionEntry_Parameter_set_option;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_SectionEntry_Parameter (MConfig_SectionEntry_Parameter const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) SectionEntry_Parameter\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("SectionEntry_Parameter\n");

    dump_mconfig_Option (el->option, nest_level + 1);

}

static void
mconfig_SectionEntry_Subsection_set_section (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_SectionEntry ||
             __el->mconfig_element_type != MConfigElement::_Section);

    MConfig_SectionEntry * const &_self = static_cast <MConfig_SectionEntry*> (__self);

    abortIf (_self->section_entry_type != MConfig_SectionEntry::_Subsection);

    MConfig_SectionEntry_Subsection * const &self = static_cast <MConfig_SectionEntry_Subsection*> (_self);

    self->section = static_cast <MConfig_Section*> (___el);
}

static ParserElement*
mconfig_SectionEntry_Subsection_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_SectionEntry_Subsection))) MConfig_SectionEntry_Subsection;
}

static Ref<Grammar>
create_mconfig_SectionEntry_Subsection ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_SectionEntry_Subsection_creation_func));
    grammar->name = String::forData ("SectionEntry_Subsection");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->grammar = create_mconfig_Section ();
        entry->assignment_func = mconfig_SectionEntry_Subsection_set_section;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_SectionEntry_Subsection (MConfig_SectionEntry_Subsection const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) SectionEntry_Subsection\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("SectionEntry_Subsection\n");

    dump_mconfig_Section (el->section, nest_level + 1);

}

static Ref<Grammar>
create_mconfig_SectionEntry ()
{
    static Ref<Grammar_Switch> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Switch ());
    grammar->name = String::forData ("SectionEntry");

    {
        Ref<SwitchGrammarEntry> entry = grab (new SwitchGrammarEntry ());
        entry->grammar = create_mconfig_SectionEntry_Parameter ();

        grammar->grammar_entries.append (entry);
    }

    {
        Ref<SwitchGrammarEntry> entry = grab (new SwitchGrammarEntry ());
        entry->grammar = create_mconfig_SectionEntry_Subsection ();

        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_SectionEntry (MConfig_SectionEntry const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) SectionEntry\n");
        return;
    }

    switch (el->section_entry_type) {
        case MConfig_SectionEntry::_Parameter:
            dump_mconfig_SectionEntry_Parameter (static_cast <MConfig_SectionEntry_Parameter const *> (el), nest_level + 0);
            break;
        case MConfig_SectionEntry::_Subsection:
            dump_mconfig_SectionEntry_Subsection (static_cast <MConfig_SectionEntry_Subsection const *> (el), nest_level + 0);
            break;
        default:
            abortIfReached ();
    }
}

static void
mconfig_SectionBody_set_sectionEntry (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_SectionBody ||
             __el->mconfig_element_type != MConfigElement::_SectionEntry);

    MConfig_SectionBody * const &self = static_cast <MConfig_SectionBody*> (__self);

    self->sectionEntrys.append (static_cast <MConfig_SectionEntry*> (__el));
}

static ParserElement*
mconfig_SectionBody_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_SectionBody))) MConfig_SectionBody;
}

static Ref<Grammar>
create_mconfig_SectionBody ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_SectionBody_creation_func));
    grammar->name = String::forData ("SectionBody");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->flags |= CompoundGrammarEntry::Optional;
        entry->flags |= CompoundGrammarEntry::Sequence;
        entry->grammar = create_mconfig_SectionEntry ();
        entry->assignment_func = mconfig_SectionBody_set_sectionEntry;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_SectionBody (MConfig_SectionBody const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) SectionBody\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("SectionBody\n");

    {
        List<MConfig_SectionEntry*>::DataIterator sub_iter (el->sectionEntrys);
        while (!sub_iter.done ()) {
            MConfig_SectionEntry *&sub = sub_iter.next ();
            dump_mconfig_SectionEntry (sub, nest_level + 1);
        }
    }

}

static void
mconfig_Section_set_name (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Section ||
             __el->mconfig_element_type != MConfigElement::_Key);

    MConfig_Section * const &self = static_cast <MConfig_Section*> (__self);

    self->name = static_cast <MConfig_Key*> (___el);
}

bool mconfig_begin_section (
        MConfig_Section *parser_element,
        Pargen::ParserControl *parser_control,
        void *data);

bool __pargen_mconfig_begin_section (
        ParserElement *parser_element,
        Pargen::ParserControl *parser_control,
        void *data)
{
    MConfigElement * const &_el =
            static_cast <MConfigElement*> (parser_element);
    abortIf (_el->mconfig_element_type != MConfigElement::_Section);

    MConfig_Section * const &el =
            static_cast <MConfig_Section*> (_el);

    return mconfig_begin_section (el, parser_control, data);
}

static void
mconfig_Section_set_sectionBody (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Section ||
             __el->mconfig_element_type != MConfigElement::_SectionBody);

    MConfig_Section * const &self = static_cast <MConfig_Section*> (__self);

    self->sectionBody = static_cast <MConfig_SectionBody*> (___el);
}

static ParserElement*
mconfig_Section_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Section))) MConfig_Section;
}

static Ref<Grammar>
create_mconfig_Section ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Section_creation_func));
    grammar->name = String::forData ("Section");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->flags |= CompoundGrammarEntry::Optional;
        entry->grammar = create_mconfig_Key ();
        entry->assignment_func = mconfig_Section_set_name;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());

        Ref<Grammar_Immediate_SingleToken> grammar__immediate = grab (new Grammar_Immediate_SingleToken ("{"));
        entry->grammar = grammar__immediate;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->inline_match_func = __pargen_mconfig_begin_section;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->grammar = create_mconfig_SectionBody ();
        entry->assignment_func = mconfig_Section_set_sectionBody;
        grammar->grammar_entries.append (entry);
    }

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());

        Ref<Grammar_Immediate_SingleToken> grammar__immediate = grab (new Grammar_Immediate_SingleToken ("}"));
        entry->grammar = grammar__immediate;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_Section (MConfig_Section const *el, Size nest_level)
{
    if (el == NULL) {
        print_tab (errf, nest_level);
        errf->print ("(null) Section\n");
        return;
    }

    print_tab (errf, nest_level);
    errf->print ("Section\n");

    dump_mconfig_Key (el->name, nest_level + 1);

    print_tab (errf, nest_level + 1);
    errf->print ("token: {\n");
    dump_mconfig_SectionBody (el->sectionBody, nest_level + 1);

    print_tab (errf, nest_level + 1);
    errf->print ("token: }\n");
}

static void
mconfig_Grammar_set_sectionEntry (
        ParserElement *___self,
        ParserElement *___el)
{
    if (___self == NULL ||
        ___el == NULL)
    {
        return;
    }

    MConfigElement * const &__self = static_cast <MConfigElement*> (___self);
    MConfigElement * const &__el = static_cast <MConfigElement*> (___el);

    abortIf (__self->mconfig_element_type != MConfigElement::_Grammar ||
             __el->mconfig_element_type != MConfigElement::_SectionEntry);

    MConfig_Grammar * const &self = static_cast <MConfig_Grammar*> (__self);

    self->sectionEntrys.append (static_cast <MConfig_SectionEntry*> (__el));
}

static ParserElement*
mconfig_Grammar_creation_func (VStack * const vstack /* non-null */)
{
    return new (vstack->push_malign (sizeof (MConfig_Grammar))) MConfig_Grammar;
}

Ref<Grammar>
create_mconfig_grammar ()
{
    static Ref<Grammar_Compound> grammar;
    if (!grammar.isNull ())
        return grammar.ptr ();

    grammar = grab (new Grammar_Compound (mconfig_Grammar_creation_func));
    grammar->name = String::forData ("Grammar");

    {
        Ref<CompoundGrammarEntry> entry = grab (new CompoundGrammarEntry ());
        entry->flags |= CompoundGrammarEntry::Optional;
        entry->flags |= CompoundGrammarEntry::Sequence;
        entry->grammar = create_mconfig_SectionEntry ();
        entry->assignment_func = mconfig_Grammar_set_sectionEntry;
        grammar->grammar_entries.append (entry);
    }

    return grammar.ptr ();
}

void
dump_mconfig_grammar (MConfig_Grammar const *el)
{
    if (el == NULL) {
        print_tab (errf, 0);
        errf->print ("(null) Grammar\n");
        return;
    }

    errf->print ("Grammar\n");

    {
        List<MConfig_SectionEntry*>::DataIterator sub_iter (el->sectionEntrys);
        while (!sub_iter.done ()) {
            MConfig_SectionEntry *&sub = sub_iter.next ();
            dump_mconfig_SectionEntry (sub, 1);
        }
    }

}

}

