#ifndef __MCONFIG__MCONFIG_PARGEN_H__
#define __MCONFIG__MCONFIG_PARGEN_H__

#include <mycpp/string.h>

#include <pargen/parser_element.h>
#include <pargen/grammar.h>

namespace MConfig {

class MConfigElement : public Pargen::ParserElement
{
public:
    enum Type {
        _Word,
        _Value,
        _Key,
        _Option,
        _SectionEntry,
        _SectionBody,
        _Section,
        _Grammar
    };

    const Type mconfig_element_type;

    MConfigElement (Type type)
        : mconfig_element_type (type)
    {
    }
};

class MConfig_Word;
class MConfig_Value;
class MConfig_Key;
class MConfig_Option;
class MConfig_SectionEntry;
class MConfig_SectionBody;
class MConfig_Section;

class MConfig_Word : public MConfigElement
{
public:
    Pargen::ParserElement_Token *any_token;

    MConfig_Word ()
        : MConfigElement (MConfigElement::_Word)
          , any_token (NULL)
    {
    }
};

void dump_mconfig_Word (MConfig_Word const *, MyCpp::Size nest_level = 0);

class MConfig_Value : public MConfigElement
{
public:
    enum Type {
        _List,
        _Word
    };

    const Type value_type;

    MConfig_Value (Type type)
        : MConfigElement (MConfigElement::_Value),
          value_type (type)
    {
    }
};

class MConfig_Value_List : public MConfig_Value
{
public:
    MyCpp::List<MConfig_Word*> words;
    MConfig_Value *value;

    MConfig_Value_List ()
        : MConfig_Value (MConfig_Value::_List)
          , value (NULL)
    {
    }
};

class MConfig_Value_Word : public MConfig_Value
{
public:
    MyCpp::List<MConfig_Word*> words;

    MConfig_Value_Word ()
        : MConfig_Value (MConfig_Value::_Word)
    {
    }
};

void dump_mconfig_Value (MConfig_Value const *, MyCpp::Size nest_level = 0);

class MConfig_Key : public MConfigElement
{
public:
    MyCpp::List<MConfig_Word*> words;

    MConfig_Key ()
        : MConfigElement (MConfigElement::_Key)
    {
    }
};

void dump_mconfig_Key (MConfig_Key const *, MyCpp::Size nest_level = 0);

bool mconfig_accept_option (
        MConfig_Option *parser_element,
        Pargen::ParserControl *parser_control,
        void *data);

class MConfig_Option : public MConfigElement
{
public:
    enum Type {
        _KeyValue,
        _Key,
        _Newline
    };

    const Type option_type;

    MConfig_Option (Type type)
        : MConfigElement (MConfigElement::_Option),
          option_type (type)
    {
    }
};

bool mconfig_accept_option (
        MConfig_Option *parser_element,
        Pargen::ParserControl *parser_control,
        void *data);

class MConfig_Option_KeyValue : public MConfig_Option
{
public:
    MConfig_Key *key;
    MConfig_Value *value;

    MConfig_Option_KeyValue ()
        : MConfig_Option (MConfig_Option::_KeyValue)
          , key (NULL)
          , value (NULL)
    {
    }
};

class MConfig_Option_Key : public MConfig_Option
{
public:
    MConfig_Key *key;

    MConfig_Option_Key ()
        : MConfig_Option (MConfig_Option::_Key)
          , key (NULL)
    {
    }
};

class MConfig_Option_Newline : public MConfig_Option
{
public:

    MConfig_Option_Newline ()
        : MConfig_Option (MConfig_Option::_Newline)
    {
    }
};

void dump_mconfig_Option (MConfig_Option const *, MyCpp::Size nest_level = 0);

class MConfig_SectionEntry : public MConfigElement
{
public:
    enum Type {
        _Parameter,
        _Subsection
    };

    const Type section_entry_type;

    MConfig_SectionEntry (Type type)
        : MConfigElement (MConfigElement::_SectionEntry),
          section_entry_type (type)
    {
    }
};

class MConfig_SectionEntry_Parameter : public MConfig_SectionEntry
{
public:
    MConfig_Option *option;

    MConfig_SectionEntry_Parameter ()
        : MConfig_SectionEntry (MConfig_SectionEntry::_Parameter)
          , option (NULL)
    {
    }
};

class MConfig_SectionEntry_Subsection : public MConfig_SectionEntry
{
public:
    MConfig_Section *section;

    MConfig_SectionEntry_Subsection ()
        : MConfig_SectionEntry (MConfig_SectionEntry::_Subsection)
          , section (NULL)
    {
    }
};

void dump_mconfig_SectionEntry (MConfig_SectionEntry const *, MyCpp::Size nest_level = 0);

class MConfig_SectionBody : public MConfigElement
{
public:
    MyCpp::List<MConfig_SectionEntry*> sectionEntrys;

    MConfig_SectionBody ()
        : MConfigElement (MConfigElement::_SectionBody)
    {
    }
};

void dump_mconfig_SectionBody (MConfig_SectionBody const *, MyCpp::Size nest_level = 0);

bool mconfig_begin_section (
        MConfig_Section *parser_element,
        Pargen::ParserControl *parser_control,
        void *data);

class MConfig_Section : public MConfigElement
{
public:
    MConfig_Key *name;
    MConfig_SectionBody *sectionBody;

    MConfig_Section ()
        : MConfigElement (MConfigElement::_Section)
          , name (NULL)
          , sectionBody (NULL)
    {
    }
};

void dump_mconfig_Section (MConfig_Section const *, MyCpp::Size nest_level = 0);

class MConfig_Grammar : public MConfigElement
{
public:
    MyCpp::List<MConfig_SectionEntry*> sectionEntrys;

    MConfig_Grammar ()
        : MConfigElement (MConfigElement::_Grammar)
    {
    }
};

MyCpp::Ref<Pargen::Grammar> create_mconfig_grammar ();

void dump_mconfig_grammar (MConfig_Grammar const *el);

}

#endif /* __MCONFIG__MCONFIG_PARGEN_H__ */

